import { makeStyles } from "@material-ui/core/styles";

export const useEditorStyles = makeStyles((theme) => ({
  statusLineWrapper: {
    marginTop: theme.spacing(0.5),
    marginBottom: theme.spacing(-1),
  },
}));
